@@ -1,4 +1,3 @@
-#!/usr/bin/env python3
 from abc import ABC, abstractmethod
 from typing import Optional, Tuple, Union
 
@@ -8,7 +7,7 @@ from torch.autograd import grad
 
 from stochman.curves import BasicCurve, CubicSpline
 from stochman.geodesic import geodesic_minimizing_energy, shooting_geodesic
-from stochman.utilities import squared_manifold_distance
+from stochman.utils import squared_manifold_distance, tensor_reduction
 
 
 class Manifold(ABC):
@@ -17,13 +16,14 @@ class Manifold(ABC):
     from this abstract base class abstraction.
     """
 
-    def curve_energy(self, curve: BasicCurve) -> torch.Tensor:
+    def curve_energy(self, curve: BasicCurve, reduction: Optional[str] = "sum") -> torch.Tensor:
         """
         Compute the discrete energy of a given curve.
 
-        Input:
-            curve:      a Nx(d) torch Tensor representing a curve or
-                        a BxNx(d) torch Tensor representing B curves.
+        Args:
+            curve: a Nx(d) torch Tensor representing a curve or a BxNx(d) torch Tensor representing B curves.
+            reduction: how to reduce the curve energy over the batch dimension. Choose between
+                `'sum'`, `'mean'`, `'none'` or `None` (where the last two will return the individual scores)
 
         Output:
             energy:     a scalar corresponding to the energy of
@@ -44,7 +44,7 @@ class Manifold(ABC):
         delta = curve[:, 1:] - curve[:, :-1]  # Bx(N-1)x(d)
         flat_delta = delta.view(-1, d)  # (B*(N-1))x(d)
         energy = self.inner(curve[:, :-1].reshape(-1, d), flat_delta, flat_delta)  # B*(N-1)
-        return energy.sum()  # scalar
+        return tensor_reduction(energy, reduction)
 
     def curve_length(self, curve: BasicCurve) -> torch.Tensor:
         """
@@ -363,11 +363,13 @@ class Manifold(ABC):
                         is the geodesic distance from p0 to p1.
         """
         if curve is None:
-            curve = self.connecting_geodesic(p0, p1)
+            curve, _ = self.connecting_geodesic(p0, p1)
         if curve is not None and optimize:
-            curve = self.connecting_geodesic(p0, p1, init_curve=curve)
+            curve, _ = self.connecting_geodesic(p0, p1, init_curve=curve)
         with torch.no_grad():
             lm = curve.deriv(torch.zeros(1))
+            if lm.ndim == 3:
+                lm.squeeze_(1)
         return lm
 
     def expmap(self, p, v):
@@ -415,13 +417,14 @@ class EmbeddedManifold(Manifold, ABC):
     should inherit from this abstract base class abstraction.
     """
 
-    def curve_energy(self, curve: BasicCurve, dt=None):
+    def curve_energy(self, curve: BasicCurve, reduction: Optional[str] = "sum", dt=None):
         """
         Compute the discrete energy of a given curve.
 
-        Input:
-            curve:      a Nx(d) torch Tensor representing a curve or
-                        a BxNx(d) torch Tensor representing B curves.
+        Args:
+            curve: a Nx(d) torch Tensor representing a curve or a BxNx(d) torch Tensor representing B curves.
+            reduction: how to reduce the curve energy over the batch dimension. Choose between
+                `'sum'`, `'mean'`, `'none'` or `None` (where the last two will return the individual scores)
 
         Output:
             energy:     a scalar corresponding to the energy of
@@ -441,9 +444,8 @@ class EmbeddedManifold(Manifold, ABC):
         emb_curve = self.embed(curve)  # BxNxD
         B, N, D = emb_curve.shape
         delta = emb_curve[:, 1:, :] - emb_curve[:, :-1, :]  # Bx(N-1)xD
-        energy = (delta ** 2).sum((1, 2)) * dt  # B
-
-        return energy
+        energy = (delta**2).sum((1, 2)) * dt  # B
+        return tensor_reduction(energy, reduction)
 
     def curve_length(self, curve: BasicCurve, dt=None):
         """
@@ -539,7 +541,7 @@ class LocalVarMetric(Manifold):
         """
         super().__init__()
         self.data = data
-        self.sigma2 = sigma ** 2
+        self.sigma2 = sigma**2
         self.rho = rho
         self.device = device
 
@@ -584,7 +586,7 @@ class LocalVarMetric(Manifold):
             if return_deriv:
                 weighted_delta = (w_p / sigma2).reshape(-1, 1).expand(-1, D) * delta  # NxD
                 dSdc = 2.0 * torch.diag(w_p.mm(delta).flatten()) - weighted_delta.t().mm(delta2)  # DxD
-                dM = dSdc.t() * (m ** 2).reshape(-1, 1).expand(-1, D)  # DxD
+                dM = dSdc.t() * (m**2).reshape(-1, 1).expand(-1, D)  # DxD
                 dMdc.append(dM.reshape(1, D, D))
 
         if return_deriv:
